@@ -1,6 +1,6 @@
 /*	MikMod sound library
-	(c) 1998, 1999 Miodrag Vallat and others - see file AUTHORS for
-	complete list.
+	(c) 1998, 1999, 2000, 2001 Miodrag Vallat and others - see file AUTHORS
+	for complete list.
 
 	This library is free software; you can redistribute it and/or modify
 	it under the terms of the GNU Library General Public License as
@@ -20,7 +20,7 @@
 
 /*==============================================================================
 
-  $Id: virtch_common.c,v 1.10 1999/06/28 03:00:06 miod Exp $
+  $Id: virtch_common.c,v 1.13 2002/01/02 04:10:39 miod Exp $
 
   Common source parts between the two software mixers.
   This file is probably the ugliest part of libmikmod...
@@ -29,7 +29,7 @@
 
 #ifndef _IN_VIRTCH_
 
-#include <mikmod_internals.h>
+#include "mikmod_internals.h"
 
 extern BOOL  VC1_Init(void);
 extern BOOL  VC2_Init(void);
@@ -103,23 +103,43 @@ extern ULONG VC1_VoiceRealVolume(UBYTE);
 extern ULONG VC2_VoiceRealVolume(UBYTE);
 static ULONG (*VC_VoiceRealVolume_ptr)(UBYTE);
 
+#if defined __STDC__ || defined _MSC_VER
 #define VC_PROC0(suffix) \
-void VC_##suffix##(void) { VC_##suffix##_ptr(); }
+MIKMODAPI void VC_##suffix (void) { VC_##suffix##_ptr(); }
 
 #define VC_FUNC0(suffix,ret) \
-ret VC_##suffix##(void) { return VC_##suffix##_ptr(); }
+MIKMODAPI ret VC_##suffix (void) { return VC_##suffix##_ptr(); }
 
 #define VC_PROC1(suffix,typ1) \
-void VC_##suffix##(typ1 a) { VC_##suffix##_ptr(a); }
+MIKMODAPI void VC_##suffix (typ1 a) { VC_##suffix##_ptr(a); }
 
 #define VC_FUNC1(suffix,ret,typ1) \
-ret VC_##suffix##(typ1 a) { return VC_##suffix##_ptr(a); }
+MIKMODAPI ret VC_##suffix (typ1 a) { return VC_##suffix##_ptr(a); }
 
 #define VC_PROC2(suffix,typ1,typ2) \
-void VC_##suffix##(typ1 a,typ2 b) { VC_##suffix##_ptr(a,b); }
+MIKMODAPI void VC_##suffix (typ1 a,typ2 b) { VC_##suffix##_ptr(a,b); }
 
 #define VC_FUNC2(suffix,ret,typ1,typ2) \
-ret VC_##suffix##(typ1 a,typ2 b) { return VC_##suffix##_ptr(a,b); }
+MIKMODAPI ret VC_##suffix (typ1 a,typ2 b) { return VC_##suffix##_ptr(a,b); }
+#else
+#define VC_PROC0(suffix) \
+MIKMODAPI void VC_/**/suffix (void) { VC_/**/suffix/**/_ptr(); }
+
+#define VC_FUNC0(suffix,ret) \
+MIKMODAPI ret VC_/**/suffix (void) { return VC_/**/suffix/**/_ptr(); }
+
+#define VC_PROC1(suffix,typ1) \
+MIKMODAPI void VC_/**/suffix (typ1 a) { VC_/**/suffix/**/_ptr(a); }
+
+#define VC_FUNC1(suffix,ret,typ1) \
+MIKMODAPI ret VC_/**/suffix (typ1 a) { return VC_/**/suffix/**/_ptr(a); }
+
+#define VC_PROC2(suffix,typ1,typ2) \
+MIKMODAPI void VC_/**/suffix (typ1 a,typ2 b) { VC_/**/suffix/**/_ptr(a,b); }
+
+#define VC_FUNC2(suffix,ret,typ1,typ2) \
+MIKMODAPI ret VC_/**/suffix (typ1 a,typ2 b) { return VC_/**/suffix/**/_ptr(a,b); }
+#endif
 
 VC_FUNC0(Init,BOOL)
 VC_PROC0(Exit)
@@ -350,6 +370,12 @@ SWORD VC1_SampleLoad(struct SAMPLOAD* sload,int type)
 		_mm_errno = MMERR_OUT_OF_HANDLES;
 		return -1;
 	}
+	
+	/* Reality check for loop settings */
+	if (s->loopend > s->length)
+		s->loopend = s->length;
+	if (s->loopstart >= s->loopend)
+		s->flags &= ~SF_LOOP;
 
 	length    = s->length;
 	loopstart = s->loopstart;
